<!DOCTYPE gui SYSTEM "kpartgui.dtd">
<gui version="5" name="akonadiconsole" >

 <MenuBar>
  <Menu name="edit">
    <Action name="akonadi_paste" append="edit_paste_merge"/>
  </Menu>
  <Menu name="collection"><text>&amp;Collection</text>
    <Action name="akonadi_collection_create"/>
    <Separator/>
    <Action name="akonadi_collection_add_to_favorites"/>
    <Action name="akonadi_collection_rename_favorite"/>
    <Action name="akonadi_collection_remove_from_favorites"/>
    <Separator/>
    <Action name="akonadi_collection_copy"/>
    <Action name="akonadi_collection_cut"/>
    <Action name="akonadi_collection_copy_to_menu"/>
    <Action name="akonadi_collection_move_to_menu"/>
    <Separator/>
    <Action name="akonadi_collection_sync"/>
    <Action name="akonadi_collection_sync_recursive"/>
    <Action name="akonadi_manage_local_subscriptions" />
    <Separator/>
    <Action name="akonadi_collection_delete"/>
    <Separator/>
    <Action name="akonadi_collection_properties"/>
  </Menu>
  <Menu name="item"><text>&amp;Item</text>
    <Action name="akonadi_item_copy"/>
    <Action name="akonadi_item_cut"/>
    <Action name="akonadi_item_copy_to_menu"/>
    <Action name="akonadi_item_move_to_menu"/>
    <Separator/>
    <Action name="akonadi_item_delete"/>
  </Menu>
  <Menu name="cache"><text>&amp;Cache</text>
    <Action name="akonadiconsole_cacheonly"/>
  </Menu>
  <Menu name="server"><text>&amp;Server</text>
    <Action name="akonadiserver_start"/>
    <Action name="akonadiserver_stop"/>
    <Action name="akonadiserver_restart"/>
    <Separator/>
    <Action name="akonadiserver_configure"/>
  </Menu>
 </MenuBar>

 <Menu name="akonadi_collectionview_contextmenu">
  <Action name="akonadi_collection_create"/>
  <Separator/>
  <Action name="akonadi_collection_add_to_favorites"/>
  <Separator/>
  <Action name="akonadi_collection_copy"/>
  <Action name="akonadi_collection_cut"/>
  <Action name="akonadi_collection_copy_to_menu"/>
  <Action name="akonadi_collection_move_to_menu"/>
  <Action name="akonadi_paste"/>
  <Separator/>
  <Action name="akonadi_collection_delete"/>
  <Separator/>
  <Action name="akonadi_collection_sync"/>
  <Action name="akonadi_collection_sync_recursive"/>
  <Action name="akonadiconsole_akonadi2xml"/>
  <Action name="akonadiconsole_clearcache"/>
  <Separator/>
  <Action name="akonadi_collection_properties"/>
 </Menu>

 <Menu name="akonadi_favoriteview_contextmenu">
  <Action name="akonadi_collection_rename_favorite"/>
  <Action name="akonadi_collection_remove_from_favorites"/>
  <Separator/>
  <Action name="akonadi_collection_copy"/>
  <Action name="akonadi_collection_cut"/>
  <Action name="akonadi_collection_copy_to_menu"/>
  <Action name="akonadi_collection_move_to_menu"/>
  <Action name="akonadi_paste"/>
  <Separator/>
  <Action name="akonadi_collection_delete"/>
  <Separator/>
  <Action name="akonadi_collection_sync"/>
  <Action name="akonadi_collection_sync_recursive"/>
  <Action name="akonadiconsole_akonadi2xml"/>
  <Separator/>
  <Action name="akonadi_collection_properties"/>
 </Menu>

 <Menu name="akonadi_itemview_contextmenu">
   <Action name="akonadi_item_copy"/>
   <Action name="akonadi_item_cut"/>
   <Action name="akonadi_item_copy_to_menu"/>
   <Action name="akonadi_item_move_to_menu"/>
   <Action name="akonadi_paste"/>
   <Action name="akonadi_item_delete"/>
 </Menu>
</gui>
